const rp = require('request-promise-native')
const defaultConfig = {
  apiServerAddress: 'http://localhost:5000'
}

function init(receivedConfig){
  const config = {
    ...defaultConfig,
    ...receivedConfig
  }

  async function doGet(route){
    const options = {
      method: 'GET',
      uri: config.apiServerAddress + route,
      json: true,
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded'
      }
    }
    try{
      return await rp(options)
    } catch(err){
      return {
        result: false,
        route,
        err
      }
    }
  }

  async function doPost(route, form={}){
    const options = {
      method: 'POST',
      uri: config.apiServerAddress + route,
      json: true,
      form,
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded',
      }
    }
    try{
      return await rp(options)
    } catch(err){
      return {
        result: false,
        route,
        err
      }
    }
  }

  async function isAPIRunning(){
    return await doGet('/isAPIRunning')
  }

  async function contractAddress(){
    return await doPost('/contractAddress')
  }

  async function country(tokenContractAddress){
    const query = {
      tokenContractAddress
    }
    return await doGet('/country', query)
  }

  async function contractIdentifier(address, tokenContractAddress){
    const query = {
      address,
      tokenContractAddress
    }
    return await doGet('/contractIdentifier', query)
  }

  async function symbol(tokenContractAddress){
    if (!tokenContractAddress) {
      return await doGet('/token/symbol')
    } else {
      return await doGet(`/tokens/${tokenContractAddress}/symbol`)
    }
  }

  async function totalSupply(tokenContractAddress){
    if (!tokenContractAddress) {
      return await doGet('/token/totalSupply')
    } else {
      return await doGet(`/tokens/${tokenContractAddress}/totalSupply`)
    }
  }

  async function generateMintTx(toAddress, cbsTxId, value, tokenContractAddress){
    const form = {
      toAddress,
      cbsTxId,
      value,
      tokenContractAddress
    }
    return await doPost('/generateMintTx', form)
  }

  async function generateMintAndHoldTx(
    toAddress,
    cbsTxId,
    value,
    otherValue,
    otherTokenContractAddress,
    counterpartyAddress,
    executeHoldAddress,
    duration,
    holdId,
    tokenContractAddress
  ){
    const form = {
      toAddress,
      cbsTxId,
      value,
      otherValue,
      otherTokenContractAddress,
      counterpartyAddress,
      executeHoldAddress,
      duration,
      holdId,
      tokenContractAddress
    }
    return await doPost('/generateMintAndHoldTx', form)
  }

  async function generateRedeemTx(cbsAccountNumber, value, tokenContractAddress){
    const form = {
      cbsAccountNumber,
      value,
      tokenContractAddress
    }
    return await doPost('/generateRedeemTx', form)
  }

  async function generateHoldForTx(
    value,
    otherValue,
    otherTokenContractAddress,
    counterpartyAddress,
    executeHoldAddress,
    duration,
    holdId,
    tokenContractAddress
  ){
    const form = {
      value,
      otherValue,
      otherTokenContractAddress,
      counterpartyAddress,
      executeHoldAddress,
      duration,
      holdId,
      tokenContractAddress
    }
    return await doPost('/generateHoldForTx', form)
  }

  async function generateExecuteHoldTx(
    counterpartyAddress,
    holdPlacerAddress,
    otherTokenAddress,
    destinationAddress,
    tokenContractAddress,
    holdId,
    otherHoldId
  ){
    const form = {
      counterpartyAddress,
      holdPlacerAddress,
      otherTokenAddress,
      destinationAddress,
      tokenContractAddress,
      holdId,
      otherHoldId
    }
    return await doPost('/generateExecuteHoldTx', form)
  }

  async function generateClearUnexpiredHoldTx(form){
    return await doPost('/generateClearUnexpiredHoldTx', form)
  }

  async function generateClearExpiredHoldsTx(
    address,
    maxNumber,
    tokenContractAddress
  ){
    const form = {
      address,
      maxNumber,
      tokenContractAddress
    }
    return await doPost('/generateClearExpiredHoldsTx', form)
  }

  async function generateTransferTx(
      toAddress,
      value,
      tokenContractAddress
  ){
    const form = {
      toAddress,
      value,
      tokenContractAddress
    }
    return await doPost('/generateTransferTx', form)
  }

  async function getBalance(address, tokenContractAddress){
    const form = {
      address,
      tokenContractAddress
    }
    return await doPost('/getBalance', form)
  }

  async function getActiveHoldsFor(address, tokenContractAddress){
    const form = {
      address,
      tokenContractAddress
    }
    return await doPost('/getActiveHoldsFor', form)
  }

  async function getActiveHoldsBy(address, tokenContractAddress){
    const form = {
      address,
      tokenContractAddress
    }
    return await doPost('/getActiveHoldsBy', form)
  }

  async function getTokenContractABI(){
    const form = {}
    return await doPost('/getTokenContractABI', form)
  }

  async function getMintEventPrototype(address){
    const form = {
      address
    }
    return await doPost('/getMintEventPrototype', form)
  }

  async function getRedemptionEventPrototype(address){
    const form = {
      address
    }
    return await doPost('/getRedemptionEventPrototype', form)
  }

  async function getTransferEventPrototype(address){
    const form = {
      address
    }
    return await doPost('/getTransferEventPrototype', form)
  }

  async function getApprovalEventPrototype(address){
    const form = {
      address
    }
    return await doPost('/getApprovalEventPrototype', form)
  }

  async function getHoldForEventPrototype(address){
    const form = {
      address
    }
    return await doPost('/getHoldForEventPrototype', form)
  }

  async function getAvailableBalanceEventPrototype(address){
    const form = {
      address
    }
    return await doPost('/getAvailableBalanceEventPrototype', form)
  }

  async function getExecuteHoldEventPrototype(address){
    const form = {
      address
    }
    return await doPost('/getExecuteHoldEventPrototype', form)
  }

  return {
    isAPIRunning,
    contractAddress,
    country,
    contractIdentifier,
    symbol,
    totalSupply,
    generateMintTx,
    generateMintAndHoldTx,
    generateRedeemTx,
    generateHoldForTx,
    generateExecuteHoldTx,
    generateClearUnexpiredHoldTx,
    generateClearExpiredHoldsTx,
    generateTransferTx,
    getActiveHoldsBy,
    getActiveHoldsFor,
    getBalance,
    getTokenContractABI,
    getMintEventPrototype,
    getRedemptionEventPrototype,
    getTransferEventPrototype,
    getApprovalEventPrototype,
    getHoldForEventPrototype,
    getAvailableBalanceEventPrototype,
    getExecuteHoldEventPrototype,
    config
  }
}

module.exports = init
